#    Diagnostic and test task for the AGATA Digitizer interface

#    Data Comms Control component 


set FADCList [list 1 2 3 4 5 6 7]
set FADC [lindex $FADCList 0]

set Value 0
set Register 0

for {set i 0} {$i <= 15} {incr i} {set gr($i) 0}
for {set i 0} {$i <= 5} {incr i} {set cr($i) 0}

    set RegisterList { \
       "Global Control" \
       "Inspection A"  \
       "Inspection B"  \
       "Inspection Enables"  \
       "Spare Allocation"  \
       "Gain & Buffer Control"  \
       "Analogue Channel Select"  \
       "Sync Phase"  \
       "Clock Control #1"   \
       "Clock Control #2"   \
       "Channel Control"  \
       "Rocket I/O Control"  \
       "Test Data Source Select"  \
       "Insert Error"   \
    }

    set RegisterAddress {  \
       {0 0}   \
       {2 0}   \
       {3 0}   \
       {4 0}   \
       {5 0}   \
       {7 0}   \
       {8 0}   \
       {11 0}  \
       {13 0}  \
       {14 0}  \
       {1 *}   \
       {3 *}   \
       {4 *}   \
       {5 *}   \
    }

 
proc Data_Comms {} {
   global env
   global Digitizer Fpga FADC FADCList
   global RegisterList

set rows 16
set cols 80

set rows1 16
set cols1 40

set rows2 16
set cols2 40


  :frame [self frame-name].comms :label "[self frame-title]: Data Communications Control" :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :message COMMSM1 :label "" :w 32
    :message COMMSM2 :label "" :w 32

     :next-row T
    :choice-stack COMMSC1 :label "Current FADC Channel" :strings $FADCList

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10

    :galley COMMSLOG1 :rows $rows1 :cols $cols1 :font fixed
    :galley COMMSLOG2 :rows $rows2 :cols $cols2 :font fixed

  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 50; :row-gap 10

    :choice-stack COMMSREGISTER :label "Register" :strings $RegisterList :rows 20
    :hskip 100
    :text COMMSVALUE  :label "Value"  :w 6 
    :button COMMSSET :label "Set Register"

    :next-row T
    :button UPDATE :command click_data_comms_update

  :panel P4 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10

    :galley COMMSLOG :rows $rows :cols $cols :font fixed

  :panel LAST :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

      :button DONE   :label "Close Window"  :command "frame-quit [self frame-name].comms"
      :message COMMSM3    :label "Last Updated"
      :message COMMSM4    :label ""

   open-frame [self frame-name].comms

  :galley_size COMMSLOG $rows $cols
  :galley_size COMMSLOG1 $rows1 $cols1
  :galley_size COMMSLOG2 $rows2 $cols2

    :show T

   Data_Comms_Update

}

proc get_fadc_register {Addr}  {
    global  Digitizer  Fpga  FpgaList  FpgaIDs
    global errorCode errorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $Addr A16} m]
        if {$rc != 0} {
            Data_Comms_InsertLog LOG  "Get returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc set_fadc_register {Addr Vl}  {
    global  Digitizer  Fpga  FpgaList  FpgaIDs
    global errorCode errorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $Addr A16 $Vl} m]
        if {$rc != 0} {
            Data_Comms_InsertLog LOG  "Set returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc Data_Comms_Update {} {
   global env
   global Digitizer Fpga FADC FADCList

   :modify COMMSM1 :label "Digitizer:-  [set Digitizer]"
   :modify COMMSM2 :label "FPGA:-  [set Fpga]"
   :modify COMMSC1 :value [lsearch $FADCList $FADC]

   :modify COMMSM4 :label "[clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

}

proc Data_Comms_RegisterReDisplay {} {
    global gr cr

    :modify COMMSLOG1 :empty T
    :modify COMMSLOG2 :empty T

    Data_Comms_InsertLog LOG1 "Global Control = [set gr(0)] ([format "0x%x" [set gr(0)]])"
    Data_Comms_InsertLog LOG1 "Global Status = [set gr(1)] ([format "0x%x" [set gr(1)]])"
    Data_Comms_InsertLog LOG1 "Inspection A = [set gr(2)] ([format "0x%x" [set gr(2)]])"
    Data_Comms_InsertLog LOG1 "Inspection B = [set gr(3)] ([format "0x%x" [set gr(3)]])"
    Data_Comms_InsertLog LOG1 "Inspection Enables = [set gr(4)] ([format "0x%x" [set gr(4)]])"
    Data_Comms_InsertLog LOG1 "Spare Allocation = [set gr(5)] ([format "0x%x" [set gr(5)]])"
    Data_Comms_InsertLog LOG1 "Optical Offset = [set gr(6)] ([format "0x%x" [set gr(6)]])"
    Data_Comms_InsertLog LOG1 "Gain & Buffer Control = [set gr(7)] ([format "0x%x" [set gr(7)]])"
    Data_Comms_InsertLog LOG1 "Analogue Channel Select = [set gr(8)] ([format "0x%x" [set gr(8)]])"
    Data_Comms_InsertLog LOG1 "Inter-Board Link Status = [set gr(9)] ([format "0x%x" [set gr(9)]])"
    Data_Comms_InsertLog LOG1 "RX Rocket I/O General Status = [set gr(10)] ([format "0x%x" [set gr(10)]])"
    Data_Comms_InsertLog LOG1 "SYNC Control = [set gr(11)] ([format "0x%x" [set gr(11)]])"
    Data_Comms_InsertLog LOG1 "SYNC Status = [set gr(12)] ([format "0x%x" [set gr(12)]])"
    Data_Comms_InsertLog LOG1 "Clock Control #1 = [set gr(13)] ([format "0x%x" [set gr(13)]])"
    Data_Comms_InsertLog LOG1 "Clock Control #2 = [set gr(14)] ([format "0x%x" [set gr(14)]])"
    Data_Comms_InsertLog LOG1 "Firmware = [set gr(15)] ([format "0x%x" [set gr(15)]])"

    Data_Comms_InsertLog LOG2 "Channel Status = [set cr(0)] ([format "0x%x" [set cr(0)]])"
    Data_Comms_InsertLog LOG2 "Channel Control = [set cr(1)] ([format "0x%x" [set cr(1)]])"
    Data_Comms_InsertLog LOG2 "Rocket I/O Status = [set cr(2)] ([format "0x%x" [set cr(2)]])"
    Data_Comms_InsertLog LOG2 "Rocket I/O Control = [set cr(3)] ([format "0x%x" [set cr(3)]])"
    Data_Comms_InsertLog LOG2 "Test Data Source Select = [set cr(4)] ([format "0x%x" [set cr(4)]])"
    Data_Comms_InsertLog LOG2 "Insert Error = [set cr(5)] ([format "0x%x" [set cr(5)]])"

}

proc Data_Comms_InsertLog {w s} {:modify COMMS$w :usingtag taginsert :strings $s\n}

proc Data_Comms_GetRegisters {} {
    global gr cr
    global Fpga FpgaIDs FpgaList FADC

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    if {[llength $FpgaIDs] == 0 || $fpga == 3 || $fpga == 12} {
        Data_Comms_InsertLog LOG "Select a Virtex FPGA"
    } else {

       for {set i 0} {$i <= 15} {incr i} {
          set vl [get_fadc_register $i]
          if {$vl == -1} {break}
          set gr($i) $vl
       }

       for {set i 0} {$i <= 5} {incr i} {
          set vl [get_fadc_register [expr ([set FADC] * 16) + [set i]]]
          if {$vl == -1} {break}
          set cr($i) $vl
       }
    }
}

proc click_data_comms_update {} {

    Data_Comms_GetRegisters
    Data_Comms_RegisterReDisplay
    Data_Comms_Update
}

proc click-commsc1 {args} {
    global FADCList FADC

   set FADC [lindex $FADCList $args]
}

proc click-commsregister {args} {
    global Register

    set Register $args
}

proc click-commsvalue {args} {
    global Value

    set Value $args
}

proc click-commsset {args} {
    global Register Value FADC
    global RegisterAddress RegisterList

    set address [lindex $RegisterAddress $Register]
    set register [lindex $RegisterList $Register]

        if {[lindex $address 1] == "*"} {
             set Addr [expr ([set FADC] * 16) + [lindex $address 0]]
        } else {
             set Addr [lindex $address 0]
        }

        Data_Comms_InsertLog LOG "Setting $register ([format "0x%x" [set Addr]]) = $Value"

        set rc [set_fadc_register $Addr $Value]
}

