#    Diagnostic and test task for the AGATA Digitizer interface


set LogFlag 1

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set AgataServerList {}
set AgataServer "select a server"
variable WebServer

set ResourceList { \
       "Core: Virtex for Segment ADC card 1" \
       "Core: Virtex for Segment ADC card 2" \
       "Core: Virtex for core ADCs" \
       "Core: Main Board" \
       "Segment: Virtex for Segment ADC card 1" \
       "Segment: Virtex for Segment ADC card 2" \
       "Segment: Virtex for Segment ADC card 3" \
       "Segment: Virtex for Segment ADC card 4" \
       "Segment: Main Board" \
}
set ResourceIDs {0 1 2 3 8 9 10 11 12}


set FpgaList {}
set FpgaIDs {}
set Fpga ""

set AddressModifierList {A16 A32}
set AddressModifier [lindex $AddressModifierList 0]

set NumDigitizer 0
set DigitizerList {}

set Digitizer -1         ;#   currently selected digitizer
set CorePresent 0        ;#   for current digitizer
set SegmentPresent 0     ;#   for current digitizer
for {set i 0} {$i <= 12} {incr i} {set Present($i) 0}     ;# index is fpga ID

set SegmentTempList { \
       "Segment: Virtex for Segment ADC card 1" \
       "Segment: Analog for Segment ADC card 1" \
       "Segment: Virtex for Segment ADC card 2" \
       "Segment: Analog for Segment ADC card 2" \
       "Segment: Virtex for Segment ADC card 3" \
       "Segment: Analog for Segment ADC card 3" \
       "Segment: Virtex for Segment ADC card 4" \
       "Segment: Analog for Segment ADC card 4" \
       "Segment: PSU #1" \
       "Segment: PSU #2" \
}

set CoreTempList { \
       "Core: Virtex for Segment ADC card 1" \
       "Core: Analog for Segment ADC card 1" \
       "Core: Virtex for Segment ADC card 2" \
       "Core: Analog for Segment ADC card 2" \
       "Core: Virtex for core ADCs" \
       "Core: Analog for core ADCs" \
       "Core: PSU #1" \
       "Core: PSU #2" \
       "Core: PSU #3" \
}


set Address 0x0
set Data 0x0
set Length 1
set BlkData 0x0000

set ShortTimeOut 5000
set verbose 0


proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode] || $FaultCode == "NONE"} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}



proc make-frame {} {
   global env
   global AddressModifierList Address Data Length BlkData

set rows 16
set cols 80

  :frame [self frame-name] :label "[self frame-title]: Diagnostics" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 0
    :text XPORTADDRT         :label "Agata Digitizer Web Service" :w 20 :value "select a server"
    :choice-stack XPORTADDRC :label "" :strings "xxxxxxxxx" 
    :hskip 100
    :checkbox VERBOSE        :label "Enable Verbose Option"
 
  :panel P3 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :choice-stack DIGITIZER :label "Digitizer"  :exclusive T :menu "" :value -1
      :button PROBE           :label "Probe Hardware" 
      :choice-stack FPGA      :label "FPGA"  :exclusive T :strings "" :value -1
      :hskip 200
      :choice-stack ADDRMOD   :label "Address Modifier"  :exclusive T :strings $AddressModifierList :value 0

  :panel P4 :layout V :padx 5 :pady 5
      :layout V; :row-gap 10; :col-gap 30
      :text ADDR     :label "Address/Command" :w 10 :value $Address
      :text DATA     :label "Set Data"    :w 10 :value $Data
      :text LENGTH   :label "Read Block Length" :w 6 :value $Length
      :text BLKDATA  :label "Write Block Data"  :w 80 :value $BlkData

  :panel P5 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :button GET    :label "Do Get"  :w 14
      :button SET    :label "Do Set"  :w 14
      :button READ   :label "Do Read Block"  :w 14
      :button WRITE  :label "Do Write Block" :w 14

  :panel P6 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :button SHUTDOWN :label "Shutdown Power" 
      :button STATUS   :label "Show Status Registers"
      :button INTERNAL :label "Select Internal Clock"
      :button EXTERNAL :label "Select External Clock"

  :panel P2 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 30
     :button CLEAR   :label "Empty Log Window" :command ":modify LOG :empty T"

  :panel P7 :layout V :padx 5 :pady 5
    :layout H; :col-gap 30
     :button COMMS   :label "Data Communications Control" :w 32 :command Data_Comms
     :button TEMP    :label "Temperature Monitoring"      :w 32 :command Monitor_Temperature
     :button TESTS   :label "Run Tests"                   :w 32 :command Hardware_Tests

   open-frame

  :galley_size LOG $rows $cols

#    focus [:path CMD].entry

   :show T
}


proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n}

proc start args {
   global started env Options AgataServerList

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

#    load application options and establish default values for any not defined

   load_Options

   if {![info exists Options(WebServer)]}     {set Options(WebServer) "localhost"}

   update_Options

   foreach option [list WebServer] {
       global $option
       set $option $Options($option)
   }

   set  AgataServerList [list $WebServer]    ;# put default into the menu
   :modify XPORTADDRC :strings $AgataServerList :value [lsearch $AgataServerList $WebServer]
   
}

proc Save_Options {} {
   global Options

   foreach option [list WebServer] {
       global $option
       set Options($option) [set $option]
   }

   update_Options
}

proc click arg {
   global history verbose longbuffer datafile usedatafile
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch -glob $w  {

      XPORTADDRT    {click-server $v}
      XPORTADDRC    {click-server $i}
      
      VERBOSE       {set verbose $v; AGATADigitizerAccessClient__AccessServerVerbose $verbose}

      FPGA          {click-fpga $i}
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click-server {v} {
    global env AgataServerList WebServer AGATADigitizerAccessServerHost
    global NumDigitizer DigitizerList Digitizer
    global verbose

    if {$v == 0} {set WebServer [first $AgataServerList]} else {set WebServer $v}
    set AgataServerList [list $WebServer]    ;# put default into the menu
    :modify XPORTADDRT :value $WebServer
    :modify XPORTADDRC :strings $AgataServerList :value [lsearch $AgataServerList $WebServer]

    Save_Options

    set AGATADigitizerAccessServerHost $WebServer
    package require AGATADigitizerAccessClient

    loadsource AGATADigitizerAccessClient     ;# better solution needed

    AGATADigitizerAccessClient__AccessServerVerbose $verbose
    set NumDigitizer [AGATADigitizerAccessClient__Init]

    insert "connected to Agata Digitizer Web Service at host $WebServer: $NumDigitizer digitizer(s) discovered"

    set DigitizerList {}
    for {set i 0} {$i < $NumDigitizer} {incr i} {lappend DigitizerList $i}
    :modify DIGITIZER :menu $DigitizerList :value -1
    
    if {[llength $DigitizerList] == 1} {click-digitizer [first $DigitizerList]} else {insert "select a digitizer"}
}

proc click-data {v} {
    global Data

    set Data $v

    insert "Data= [format "0x%x" $Data]"
}

proc click-blkdata {v} {
    global BlkData

    set BlkData $v
}

proc click-length {v} {
    global Length

    set Length $v

    insert "Read Block Length= [format "0x%x" $Length]"
}


proc click-addr {v} {
    global Address

    set Address $v

    insert "Address/Cmd= [format "0x%x" $Address]"
}

proc click-probe {args} {
    global Digitizer

    click-digitizer $Digitizer
}

proc click-digitizer {v} {
    global Digitizer DigitizerList Present CorePresent SegmentPresent
    global FpgaList FpgaIDs
    global ShortTimeOut

    set Digitizer $v
    set CorePresent 0
    set SegmentPresent 0
    for {set i 0} {$i <= 12} {incr i} {set Present($i) 0} 

    insert "Digitizer= $Digitizer"

    set FpgaList {}
    set FpgaIDs {}

    insert "probing for Core Main Board"

    set fpga 3     ;# core main board
    set cmd 14     ;# safe read command for test
    set Length 6   ;# returns 6 bytes
    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
    if {$rc == 0} {
        insert "found it"
        lappend FpgaList "Core: Main Board"
        lappend FpgaIDs $fpga
        set CorePresent 1
        set Present($fpga) 1
    } else {
        insert "Error returned [first [split $m \n]]"
     }

    if {$CorePresent == 1} {

       insert "probing for Core ADCs"

       set fpga 2    ;# core ADCs
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Core: Virtex for core ADCs"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Core segment ADC card #1"

click-status

       set fpga 0    ;# core segment ADC card #1
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Core: Virtex for Segment ADC card 1"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

click-status

       insert "probing for Core segment ADC card #2"

       set fpga 1    ;# core segment ADC card #2
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Core: Virtex for Segment ADC card 2"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }
    }

    insert "probing for Segment Main Board"

    set fpga 12    ;# segment main board
    set cmd 14     ;# safe read command for test
    set Length 6   ;# returns 6 bytes
    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
    if {$rc == 0} {
        insert "found it"
        lappend FpgaList "Segment: Main Board"
        lappend FpgaIDs $fpga
        set SegmentPresent 1
        set Present($fpga) 1
    } else {
        insert "Error returned [first [split $m \n]]"
    }

    if {$SegmentPresent == 1} {

       insert "probing for Segment ADC card #1"

       set fpga 8    ;# Segment ADC card #1
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 1"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Segment ADC card #2"

       set fpga 9    ;# Segment ADC card #2
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 2"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Segment ADC card #3"

       set fpga 10    ;# Segment ADC card #3
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 3"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Segment ADC card #4"

       set fpga 11    ;# Segment ADC card #4
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 4"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }
   }

   set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer 0} m]   ;# reset timeout to default

   if {[expr $CorePresent + $SegmentPresent] == 0} {
       insert "Digitizer not found"
       :modify DIGITIZER  :value -1
       :modify FPGA :strings "" :value -1
   } else {
       :modify DIGITIZER :value [lsearch $DigitizerList $Digitizer]
       :modify FPGA :strings $FpgaList :value 0
       click-fpga [first $FpgaList]
   }

}

proc click-fpga {v} {
    global Fpga

    set Fpga $v

    insert "Fpga= $Fpga"
}

proc click-addrmod {v} {
    global AddressModifierList AddressModifier

    set AddressModifier [lindex $AddressModifierList $v]

    insert "AddressModifier= $AddressModifier"
}

proc click-status {args} {
    global Digitizer FpgaList FpgaIDs Fpga
    global ShortTimeOut
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
    if {$fpga < 8} then {set fpga 3} else {set fpga 12} ;# select main board 

    set cmd 14     ;# read status registers command
    set Length 6   ;# returns 6 bytes
    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
    if {$rc == 0} {
        insert "status registers: $m"
    } else {
        insert "Read returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    }
}

proc click-clock {vl} {
    global Digitizer FpgaList FpgaIDs Fpga
    global ShortTimeOut
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga 3       ;# available to core main board only
    set cmd 40 

    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $cmd A16 $vl} m]
    if {$rc == 0} {
        insert "Done"
    } else {
        insert "Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    }
}

proc click-internal {args} {click-clock 0x0100}
proc click-external {args} {click-clock 0x0000}

proc click-shutdown {args} {
    global Digitizer FpgaList FpgaIDs Fpga
    global ShortTimeOut
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
    if {$fpga < 8} then {set fpga 3} else {set fpga 12} ;# select main board 

    set cmd 20       ;# shutdown command

    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $cmd A16 0x0800} m]
    if {$rc == 0} {
        insert "Done"
    } else {
        insert "Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    }
}

proc click-set {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address AddressModifier Data
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $Address $AddressModifier $Data} m]
    if {$rc != 0} {
        insert "Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Set returned $m ([format "0x%x" $m])"
    }
}

proc click-get {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $Address $AddressModifier} m]
    if {$rc != 0} {
        insert "Get returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Get returned $m ([format "0x%x" $m])"
    }
}

proc click-read {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $Address $AddressModifier $Length} m]
    if {$rc != 0} {
        insert "Read returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Read returned $m"
    }
}

proc click-write {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier BlkData
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Write $Digitizer $fpga $Address $AddressModifier [llength $BlkData] $BlkData} m]
    if {$rc != 0} {
        insert "Write returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Write returned $m"
    }
}

